import * as ec2 from '@aws-cdk/aws-ec2';
import * as ecs from '@aws-cdk/aws-ecs';
import { Schedule } from '@aws-cdk/aws-events';
import { RetentionDays } from '@aws-cdk/aws-logs';
import { Construct } from '@aws-cdk/core';
import * as cr from '@aws-cdk/custom-resources';
/**
 * Fargate platform version.
 *
 * @stability stable
 */
export declare enum PlatformVersion {
    /**
     * @stability stable
     */
    V1_3_0 = "1.3.0",
    /**
     * @stability stable
     */
    V1_4_0 = "1.4.0",
    /**
     * @stability stable
     */
    LATEST = "LATEST"
}
/**
 * @stability stable
 */
export declare enum LaunchType {
    /**
     * @stability stable
     */
    FARGATE = "FARGATE",
    /**
     * @stability stable
     */
    EXTERNAL = "EXTERNAL"
}
/**
 * @stability stable
 */
export interface RunTaskProps {
    /**
     * The VPC for the Amazon ECS task.
     *
     * @default - create a new VPC or use existing one
     * @stability stable
     */
    readonly vpc?: ec2.IVpc;
    /**
     * The subnets to run the task.
     *
     * @default - { ec2.SubnetType.PRIVATE_WITH_NAT }
     * @stability stable
     */
    readonly vpcSubnets?: ec2.SubnetSelection;
    /**
     * The Amazon ECS Cluster.
     *
     * @default - create a new cluster
     * @stability stable
     */
    readonly cluster?: ecs.ICluster;
    /**
     * The Amazon ECS Task definition for AWS Fargate.
     *
     * @stability stable
     */
    readonly task: ecs.FargateTaskDefinition;
    /**
     * run it at once(immediately after deployment).
     *
     * @default true
     * @stability stable
     */
    readonly runAtOnce?: boolean;
    /**
     * run the task again on the custom resource update.
     *
     * @default false
     * @stability stable
     */
    readonly runOnResourceUpdate?: boolean;
    /**
     * run the task with defined schedule.
     *
     * @default - no shedule
     * @stability stable
     */
    readonly schedule?: Schedule;
    /**
     * Log retention days.
     *
     * @default - one week
     * @stability stable
     */
    readonly logRetention?: RetentionDays;
    /**
     * Fargate platform version.
     *
     * @default LATEST
     * @stability stable
     */
    readonly fargatePlatformVersion?: PlatformVersion;
    /**
     * fargate security group.
     *
     * @default - create a default security group
     * @stability stable
     */
    readonly securityGroup?: ec2.ISecurityGroup;
    /**
     * The capacity provider strategy to run the fargate task;
     *
     * @default - No capacity provider strategy defined. Use LaunchType instead.
     * @stability stable
     */
    readonly capacityProviderStrategy?: ecs.CapacityProviderStrategy[];
    /**
     * Luanch Type of the task.
     *
     * Default is `FARGATE`, however, if you choose `EXTERNAL`, you are allowed to
     * run external tasks on external instances. The `capacityProviderStrategy` will be ingored if you specify
     * this property. Please note this is the feature from ECS Anywhere. The external task will be scheduled on the
     * registered external instance(s). No fargate task will be scheduled in the `EXTERNAL` launch type.
     *
     * @default FARGATE
     * @stability stable
     */
    readonly launchType?: LaunchType;
}
/**
 * @stability stable
 */
export declare class RunTask extends Construct implements ec2.IConnectable {
    /**
     * @stability stable
     */
    readonly vpc: ec2.IVpc;
    /**
     * @stability stable
     */
    readonly cluster: ecs.ICluster;
    /**
     * The custom resource of the runOnce execution.
     *
     * @stability stable
     */
    readonly runOnceResource?: cr.AwsCustomResource;
    /**
     * fargate task security group.
     *
     * @stability stable
     */
    readonly securityGroup: ec2.ISecurityGroup;
    /**
     * makes RunTask "connectable".
     *
     * @stability stable
     */
    readonly connections: ec2.Connections;
    /**
     * @stability stable
     */
    constructor(scope: Construct, id: string, props: RunTaskProps);
}
